@@ -15,7 +15,8 @@ from typing import Any, Optional, Union
 
 from dateutil import rrule
 from ical.timespan import Timespan
-from pydantic import BaseModel, Field, root_validator, validator
+from ical.types.recur import Recur
+from pydantic import BaseModel, Field, root_validator
 
 __all__ = [
     "Calendar",
@@ -26,16 +27,39 @@ __all__ = [
     "VisibilityEnum",
     "ResponseStatus",
     "Attendee",
+    "AccessRole",
 ]
 
 _LOGGER = logging.getLogger(__name__)
 
 DATE_STR_FORMAT = "%Y-%m-%d"
 EVENT_FIELDS = (
-    "id,summary,description,location,start,end,transparency,status,eventType,"
+    "id,iCalUID,summary,start,end,description,location,transparency,status,eventType,"
     "visibility,attendees,attendeesOmitted,recurrence,recurringEventId,originalStartTime"
 )
 MIDNIGHT = datetime.time()
+ID_DELIM = "_"
+
+
+class AccessRole(str, Enum):
+    """The effective access role of the caller."""
+
+    FREE_BUSY_READER = "freeBusyReader"
+    """Provides read access to free/busy information."""
+
+    READER = "reader"
+    """Provides read access to the calendar."""
+
+    WRITER = "writer"
+    """Provides read and write access to the calendar."""
+
+    OWNER = "owner"
+    """Provides ownership of the calendar."""
+
+    @property
+    def is_writer(self) -> bool:
+        """Return if this role can create, delete, update events."""
+        return self in (AccessRole.WRITER, AccessRole.OWNER)
 
 
 class Calendar(BaseModel):
@@ -56,6 +80,20 @@ class Calendar(BaseModel):
     timezone: Optional[str] = Field(alias="timeZone", default=None)
     """The time zone of the calendar."""
 
+    access_role: AccessRole = Field(alias="accessRole")
+    """The effective access role that the authenticated user has on the calendar."""
+
+    selected: bool = False
+    """Whether the calendar content shows up in the calendar UI."""
+
+    primary: bool = False
+    """Whether the calendar is the primary calendar of the authenticated user."""
+
+    class Config:
+        """Pydnatic model configuration."""
+
+        allow_population_by_field_name = True
+
 
 class DateOrDatetime(BaseModel):
     """A date or datetime."""
@@ -79,23 +117,12 @@ class DateOrDatetime(BaseModel):
 
     @property
     def value(self) -> Union[datetime.date, datetime.datetime]:
-        """Return either a datetime or date representing the Datetime.
-
-        This only replace tzinfo/offset in the datetime when the API response did not specify one
-        separately, preferring the offset specified in the event response.
-        """
+        """Return either a datetime or date representing the Datetime."""
         if self.date is not None:
             return self.date
         if self.date_time is not None:
-            if self.timezone is not None:
-                # Always use the timezone when there isn't one, otherwise only override when
-                # using to start recurring events. Otherwise, just use the simple offset
-                # specified in the event.
-                if self.date_time.tzinfo is None:
-                    return self.date_time.replace(
-                        tzinfo=zoneinfo.ZoneInfo(self.timezone)
-                    )
-                return self.date_time.astimezone(tz=zoneinfo.ZoneInfo(self.timezone))
+            if self.date_time.tzinfo is None and self.timezone is not None:
+                return self.date_time.replace(tzinfo=zoneinfo.ZoneInfo(self.timezone))
             return self.date_time
         raise ValueError("Datetime has invalid state with no date or date_time")
 
@@ -207,6 +234,82 @@ class Attendee(BaseModel):
     """The attendee's response status."""
 
 
+class SyntheticEventId:
+    """Used to generate a event ids for synthetic recurring events.
+
+    A `gcal_sync.timeline.Timeline` will create synthetic events for each instance
+    of a recurring event. The API returns the original event id of the underlying
+    event as `recurring_event_id`. This class is used to create the synthetic
+    unique `event_id` that includes the date or datetime value of the event instance.
+
+    This class does not generate values in the `recurring_event_id` field.
+    """
+
+    def __init__(
+        self, event_id: str, dtstart: datetime.date | datetime.datetime
+    ) -> None:
+        self._event_id = event_id
+        self._dtstart = dtstart
+
+    @classmethod
+    def of(  # pylint: disable=invalid-name]
+        cls,
+        event_id: str,
+        dtstart: datetime.date | datetime.datetime,
+    ) -> SyntheticEventId:
+        """Create a SyntheticEventId based on the event instance."""
+        return SyntheticEventId(event_id, dtstart)
+
+    @classmethod
+    def parse(cls, synthetic_event_id: str) -> SyntheticEventId:
+        """Parse a SyntheticEventId from the event id string."""
+        parts = synthetic_event_id.rsplit(ID_DELIM, maxsplit=1)
+        if len(parts) != 2:
+            raise ValueError(
+                f"id was not a valid synthetic_event_id: {synthetic_event_id}"
+            )
+        dtstart: datetime.date | datetime.datetime
+        if len(parts[1]) != 8:
+            if len(parts[1]) == 0 or parts[1][-1] != "Z":
+                raise ValueError(
+                    f"SyntheticEventId had invalid date/time or timezone: {synthetic_event_id}"
+                )
+
+            dtstart = datetime.datetime.strptime(
+                parts[1][:-1], "%Y%m%dT%H%M%S"
+            ).replace(tzinfo=datetime.timezone.utc)
+        else:
+            dtstart = datetime.datetime.strptime(parts[1], "%Y%m%d").date()
+        return SyntheticEventId(parts[0], dtstart)
+
+    @classmethod
+    def is_valid(cls, synthetic_event_id: str) -> bool:
+        """Return true if the value is a valid SyntheticEventId string."""
+        try:
+            cls.parse(synthetic_event_id)
+        except ValueError:
+            return False
+        return True
+
+    @property
+    def event_id(self) -> str:
+        """Return the string value of the new event id."""
+        if isinstance(self._dtstart, datetime.datetime):
+            utc = self._dtstart.astimezone(datetime.timezone.utc)
+            return f"{self._event_id}{ID_DELIM}{utc.strftime('%Y%m%dT%H%M%SZ')}"
+        return f"{self._event_id}{ID_DELIM}{self._dtstart.strftime('%Y%m%d')}"
+
+    @property
+    def original_event_id(self) -> str:
+        """Return the underlying/original event id."""
+        return self._event_id
+
+    @property
+    def dtstart(self) -> datetime.date | datetime.datetime:
+        """Return the date value for the event id."""
+        return self._dtstart
+
+
 class Event(BaseModel):
     """A single event on a calendar."""
 
@@ -214,7 +317,12 @@ class Event(BaseModel):
     """Opaque identifier of the event."""
 
     ical_uuid: Optional[str] = Field(alias="iCalUID", default=None)
-    """Event unique identifier as defined in RFC5545."""
+    """Event unique identifier as defined in RFC5545.
+
+    Note that the iCalUID and the id are not identical. One difference in
+    their semantics is that in recurring events, all occurrences of one event
+    have different ids while they all share the same iCalUIDs.
+    """
 
     summary: str = ""
     """Title of the event."""
@@ -238,11 +346,13 @@ class Event(BaseModel):
     calendar or `transparent` which means it does not block time on the calendar.
     """
 
-    # Note deleted events are only returned in some scenarios based on request options
-    # such as enabling incremental sync or explicitly asking for deleted items. That is,
-    # most users should not need to check the status.
     status: EventStatusEnum = EventStatusEnum.CONFIRMED
-    """Status of the event."""
+    """Status of the event.
+
+    Note that deleted events are only returned in some scenarios based on request options
+    such as enabling incremental sync or explicitly asking for deleted items. That is,
+    most use cases should not need to involve checking the status.
+    """
 
     event_type: EventTypeEnum = Field(alias="eventType", default=EventTypeEnum.DEFAULT)
     """Specific type of the event."""
@@ -284,6 +394,13 @@ class Event(BaseModel):
                 f"Invalid recurrence rule: {self.json()}: {str(err)}"
             ) from err
 
+    @property
+    def recur(self) -> Recur:
+        """Build a recurrence rule for the event."""
+        if len(self.recurrence) != 1:
+            raise ValueError(f"Unexpected recurrence value: {self.recurrence}")
+        return Recur.from_rrule(self.recurrence[0])
+
     @root_validator(pre=True)
     def _allow_cancelled_events(cls, values: dict[str, Any]) -> dict[str, Any]:
         """Special case for canceled event tombstones that are missing required fields."""
@@ -303,18 +420,6 @@ class Event(BaseModel):
                 values["visibility"] = "private"
         return values
 
-    @validator("recurrence", each_item=True)
-    def _validate_rrule_params(cls, rule: str) -> str:
-        """Remove rrule property parameters not supported by the dateutil.rrule library."""
-        if not rule.startswith("RRULE;"):
-            return rule
-        right = rule[6:]
-        parts = right.split(":", maxsplit=1)
-        if len(parts) == 2:
-            # Rebuild string without parameters
-            return f"RRULE:{parts[1]}"
-        return rule  # rrule parser fail
-
     @root_validator
     def _validate_rrule(cls, values: dict[str, Any]) -> dict[str, Any]:
         """The API returns invalid RRULEs that need to be coerced to valid."""
